/* All Rights reserved */

#include <AppKit/AppKit.h>
#include "Clock.h"
#include <math.h>

@interface AppController : NSObject
{
  id faceColorW;
  id ampmSwitch;
  id shadowSwitch;
  id transSlider;
  id handColorW;
  id secColorW;
  id frameColorW;
  id prefPanel;
  id markColorW;
  id freqText;
  id secondSwitch;
  id numberPopUp;
  id alarmWindow;

  Clock *_clock;
  Clock *bigClock;
  NSTimer *timer;
  BOOL doFloor;
}
- (void) openPreferences: (id)sender;
- (void) setFrameColor: (id)sender;
- (void) setFaceColor: (id)sender;
- (void) setFaceTransparency: (id)sender;
- (void) setShowAMPM: (id)sender;
- (void) setShadow:(id)sender;
- (void) setSecondHandColor: (id)sender;
- (void) setHandColor: (id)sender;
- (void) setMarkColor: (id)sender;
- (void) setFrequency: (id)sender;
- (void) setSecond: (id)sender;
- (void) setNumberType: (id)sender;
@end
